#ifndef _Command_CPP
#define _Command_CPP
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <Windows.H>
#include <WindowsX.H>
#include <ShellAPI.H>
#include <Stdio.H>
#include <Stdlib.H>

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "../Resources/Resource.H"

#include "../../SharedSource/LZARICode.H"
#include "../../SharedSource/RLECompression.H"

#include "../../SharedClasses/CRC32/CRC.H"
#include "../../SharedClasses/SQLClass/cRecordSet.H"
#include "../../SharedClasses/SQLClass/cSQL.H"
#include "../../SharedClasses/LZSS/LZSS_Compress.H"
#include "../../SharedClasses/LZSS/LZSS_Uncompress.H"

#include "../CSockSrvr/CSockSrvr.H"

#include "../../SharedSource/NSWFL.H"
#include "Init.H"
#include "Entry.H"
#include "Routines.H"
#include "Command.H"
#include "SQLExport.H"
#include "Console.H"
#include "DBInit.H"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool ConnectToServer(bool bForcedConnect)
{
	char IPAddress[64];
	char sSQL[1024];
	CRecordSet rsTemp;
	
	if(gServer.bcConnected[CCI.iThisClient])
	{
		WriteSysLogEx("Failed to connect to the SQL-Exchange server: Already connected.", EVENT_WARN);
		return false;
	}

	if(!GetIPAddress(gsServerAddress, IPAddress))
    {
		WriteSysLogEx("Failed to connect to the SQL-Exchange server: Error resolving hostname.", EVENT_ERROR);
        return false;
    }

	if(CCI.cCustSQL.Connect(gsSQLDriver, gsSQLServer, gsSQLUserID, gsSQLPassword, "Master"))
	{
		WriteSysLogEx("Successfully connected to the SQL Master database.", EVENT_NONE);
		CCI.bCustDBConnected = true;
	}
	else{
		WriteSysLogEx("Failed to connected to the SQL Master database.", EVENT_ERROR);
		return false;						
	}

	if(!bForcedConnect)
	{
		sprintf(sSQL, "SELECT TransTable"
			" FROM [%s].[%s].SQLExch_Trans",
			gsReplicationDB, gsDefaultDBO);

		if(!CCI.cCustSQL.Execute(sSQL, &rsTemp))
		{
			WriteSysLogEx("Failed to get the pending transaction count.", EVENT_ERROR);

			rsTemp.Close();
			CCI.cCustSQL.Disconnect();
			return false;
		}

		if(rsTemp.RowCount <= 0)
		{
			WriteSysLogEx("No new transactions are available, no need to connect.", EVENT_NONE);
			rsTemp.Close();
			CCI.cCustSQL.Disconnect();
			return true;
		}
	}

	if(gServer.DoConnect(IPAddress, giServerPort, &CCI.iThisClient))
	{
		WriteSysLogEx("Connected successfully to the SQL-Exchange server.", EVENT_NONE);
	}
	else{
		WriteSysLogEx("Failed to connect to the SQL-Exchange server.", EVENT_WARN);

		if(gServer.bcConnected[CCI.iThisClient])
		{
			WriteSysLogEx("Dropping a possible locked connection.", EVENT_WARN);
			gServer.bcDisconnect[CCI.iThisClient] = true;
		}

		CCI.cCustSQL.Disconnect();
		return false;
	}

	return true;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
	int ProcessCommand(CSockSrvr *pSockSrvr, int iClient, char *sRecvBuf, int iRecvBufSz);

	Processes a command from the peer.

	Possible return Values:
		CMD_OK    The command was a success, all is well.
		CMD_DONE  The command was a success, close the connection.
		CMD_ERROR The command failed due to an error.
*/
int ProcessCommand(CSockSrvr *pSockSrvr, int iClient, char *sCmdBuf, int iCmdBufSz)
{
    char sCmdData[SENDBUFSZ + RECVBUFSZ];
	char sTemp[1024];

	int iSendBufSz = 0;
	int iCmdDataSz = 0;
	int iCmdFlagLength = 0;

	if((iCmdFlagLength = CmdCmp(sCmdBuf, "::BeginInit")))
    {
		WriteLogEx(pSockSrvr->icClientID[iClient], "Server is requesting that the replication be initialized.", EVENT_INFO);

		char sTempFile[MAX_PATH];

		GetTempFileName(sTempFilesPath, "SEC", 0, sTempFile);

		if(GenerateDBCreationSQL(&CCI.cCustSQL, pSockSrvr, iClient, sTempFile))
		{
			pSockSrvr->SetNextSendData(iClient, "::SendingDBCreationSQL");
			SendFileData(pSockSrvr, iClient, sTempFile, false);
		}

		DeleteFile(sTempFile);

		return CMD_OK;
	}
	else if((iCmdFlagLength = CmdCmp(sCmdBuf, "::BeginReplication->")))
    {
		if(gbDebugMode)
		{
			WriteLogEx(pSockSrvr->icClientID[iClient], "Received the begin transaction.", EVENT_NONE);
		}

		iCmdDataSz = BreakCmdFromData(sCmdBuf, iCmdFlagLength, iCmdBufSz, sCmdData);

		if( strcmpi(sCmdData, "Transactions") == 0)
		{
			//Need to set the "SQLExch_Trans.SQLExch_Pending" flag to 1
			//	so that we dont delete any new transaction when we clear
			//	the table after the replication process.
			sprintf(sTemp, "UPDATE [%s].[%s].SQLExch_Trans"
				" SET SQLExch_Pending = 1",
				gsReplicationDB, gsDefaultDBO);
			CCI.cCustSQL.ExecuteNonQuery(sTemp);

			//Get a list of all of the statements that need to be ran.
			sprintf(sTemp, "SELECT DISTINCT A.Statement, A.OnSuccess, A.OnFailure, A.TransDB, A.ImportTable, A.[Sequence]"
				" FROM [%s].[%s].SQLExch_Statements AS A, [%s].[%s].SQLExch_Trans AS B"
				" WHERE A.TransDB = B.TransDB"
				" AND B.TransTable = A.TransTable"
				" AND A.Active = 1 AND B.SQLExch_Pending = 1"
				" ORDER BY A.[Sequence], A.Statement, A.OnSuccess, A.OnFailure, A.ImportTable",
				gsReplicationDB, gsDefaultDBO,
				gsReplicationDB, gsDefaultDBO);
		}
		else if( strcmpi(sCmdData, "All") == 0)
		{
			//Get a list of all of the statements that need to be ran.
			sprintf(sTemp, "SELECT DISTINCT 'SELECT * FROM"
				" [' + TransDB + '].[%s].[' + TransTable + ']', NULL, NULL, TransDB, TransTable"
				" FROM [%s].[%s].SQLEXCH_Statements",
				gsDefaultDBO, gsReplicationDB, gsDefaultDBO);
		}

		CCI.rsTrans.bReplaceSingleQuotes = false;
		if(!CCI.cCustSQL.Execute(sTemp, &CCI.rsTrans))
		{
			WriteLogEx(pSockSrvr->icClientID[iClient], "Failed to get a list of pending transactions.", EVENT_ERROR);
			return CMD_ERROR;
		}

		sprintf(sTemp, "Processing %d replication statements.", CCI.rsTrans.RowCount);
		WriteLogEx(pSockSrvr->icClientID[iClient], sTemp, EVENT_NONE);

		return ExecuteTransactionSQL(pSockSrvr, iClient);
	}
	else if((iCmdFlagLength = CmdCmp(sCmdBuf, "::CompressionLevel->")))
    {
		if(gbDebugMode)
		{
			WriteLogEx(pSockSrvr->icClientID[iClient], "Received compression level option.", EVENT_NONE);
		}

		iCmdDataSz = BreakCmdFromData(sCmdBuf, iCmdFlagLength, iCmdBufSz, sCmdData);

		CCI.iCompressionLevel = atol(sCmdData);

		if(CCI.iCompressionLevel < 0)
		{
			CCI.iCompressionLevel = 0;
		}

		return CMD_OK;
	}
	else if((iCmdFlagLength = CmdCmp(sCmdBuf, "::CompressionMethod->")))
    {
		if(gbDebugMode)
		{
			WriteLogEx(pSockSrvr->icClientID[iClient], "Received compression option.", EVENT_NONE);
		}

		iCmdDataSz = BreakCmdFromData(sCmdBuf, iCmdFlagLength, iCmdBufSz, sCmdData);

		if(strlen(sCmdData) > 0 && strcmpi(sCmdData, "None") != 0)
		{
			strcpy(CCI.sCompressionMethod, sCmdData);
			CCI.bUseCompression = true;
		}
		else{
			WriteLogEx(pSockSrvr->icClientID[iClient], "Compression is off. This may overwork the network.", EVENT_WARN);
			CCI.bUseCompression = false;
		}

		return CMD_OK;
	}
	else if((iCmdFlagLength = CmdCmp(sCmdBuf, "::Msg->")))
    {
		if(gbDebugMode)
		{
			WriteLogEx(pSockSrvr->icClientID[iClient], "Received a message from the server.", EVENT_NONE);
		}

		iCmdDataSz = BreakCmdFromData(sCmdBuf, iCmdFlagLength, iCmdBufSz, sCmdData);
		WriteLogEx(pSockSrvr->icClientID[iClient], sCmdData, EVENT_NONE);

		WriteLogEx(pSockSrvr->icClientID[iClient], sCmdData, EVENT_INFO);

		return CMD_OK;
	}
    else if((iCmdFlagLength = CmdCmp(sCmdBuf, "::LastImportSuccess")))
    {
		//The last import succeeded. If we need to take action then this is where to do it.
		WriteLogEx(pSockSrvr->icClientID[iClient], "Last import was a Success.", EVENT_NONE);

		bool bResult = false;

		if(strlen(CCI.sOnSuccess) > 0)
		{
			bResult = CCI.cCustSQL.ExecuteNonQuery(CCI.sOnSuccess);
			strcpy(CCI.sOnSuccess, "");
			if(!bResult)
			{
				WriteLogEx(pSockSrvr->icClientID[iClient], "Failed to execute the OnSuccess statement.", EVENT_ERROR);
				return CMD_ERROR;
			}
		}

		return ExecuteTransactionSQL(pSockSrvr, iClient);
	}
    else if((iCmdFlagLength = CmdCmp(sCmdBuf, "::LastImportFailed")))
    {
		//The last import failed. If we need to take action then this is where to do it.

		bool bResult = false;

		WriteLogEx(pSockSrvr->icClientID[iClient], "Last SQL import failed.", EVENT_ERROR);

		if(strlen(CCI.sOnFailure) > 0)
		{
			bResult = CCI.cCustSQL.ExecuteNonQuery(CCI.sOnFailure);
			strcpy(CCI.sOnFailure, "");
			if(!bResult)
			{
				WriteLogEx(pSockSrvr->icClientID[iClient], "Failed to execute the OnFailure statement.", EVENT_ERROR);
				return CMD_ERROR;
			}
		}

		if(ExecuteTransactionSQL(pSockSrvr, iClient))
		{
			return CMD_OK;
		}
		else{
			return CMD_ERROR;
		}

		return CMD_OK;
	}

	WriteLogEx(pSockSrvr->icClientID[iClient], "Received unknown command.", EVENT_ERROR);
	return CMD_ERROR;	
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool SendFileData(CSockSrvr *pSockSrvr, int iClient, char *sFileName, bool bSendCRC)
{
    FILE *hSourceHandle = NULL;

	char sFullFile[MAX_PATH];
    char sFileBuffer[FILEBUFFERSZ + 1];
	int iBytesRead = 0;

    DWORD dwDataCRC = 0xffffffff;

	if(CCI.bUseCompression)
	{
		float dwBeforeSz = (float) Get_FileSize(sFileName);
		float dwAfterSz = 0;

		sprintf(sFullFile, "%s.sec", sFileName);

		sprintf(sFileBuffer, "Compressing file. (%.2f KB)", dwBeforeSz / 1024);
		WriteLogEx(pSockSrvr->icClientID[iClient], sFileBuffer, EVENT_NONE);
		
		if(strcmpi(CCI.sCompressionMethod, "LZSS") == 0){
			LZSSCompressFile(sFileName, sFullFile, CCI.iCompressionLevel);
		}
		else if(strcmpi(CCI.sCompressionMethod, "LZARI") == 0){
			CompressFile(sFileName, sFullFile);
		}
		else if(strcmpi(CCI.sCompressionMethod, "RLE") == 0){
			RLEEncodeFile(sFileName, sFullFile);
		}

		dwAfterSz = (float) Get_FileSize(sFullFile);

		sprintf(sFileBuffer, "File size reduced from %.2f KB to %.2f KB. (%.2f %% Compression).",
			dwBeforeSz / 1024, dwAfterSz / 1024, (100 - ((dwAfterSz / dwBeforeSz) * 100)));
		WriteLogEx(pSockSrvr->icClientID[iClient], sFileBuffer, EVENT_NONE);
	}
	else{
		strcpy(sFullFile, sFileName);
	}

	WriteLogEx(pSockSrvr->icClientID[iClient], "Begining file transfer.", EVENT_NONE);
	if( (hSourceHandle = fopen(sFullFile, "rb")) == NULL)
    {
		WriteLogEx(pSockSrvr->icClientID[iClient], "Failed to open export file for binary read.", EVENT_ERROR);
        return false;
    }

    do{
        //if(MyClient.PeekSizeOfSendData() == 0)
        //{
            iBytesRead = fread(sFileBuffer, sizeof(char), FILEBUFFERSZ, hSourceHandle);
			if(bSendCRC)
			{
	            dwDataCRC = PartialCRC(dwDataCRC, sFileBuffer, iBytesRead);
			}
            Encode(&CCI.SC, sFileBuffer, sFileBuffer, iBytesRead);
			pSockSrvr->SetNextSendDataEx(iClient, sFileBuffer, iBytesRead);
        //}
        //else Sleep(1);
    } while(iBytesRead == FILEBUFFERSZ && pSockSrvr->bcConnected[iClient] && !pSockSrvr->bcDisconnect[iClient]);

	if(bSendCRC)
	{
		dwDataCRC = (dwDataCRC ^ 0xffffffff);
	}
	
	fclose(hSourceHandle);

	DeleteFile(sFullFile);
	DeleteFile(sFileName);

	if(pSockSrvr->bcConnected[iClient] && !pSockSrvr->bcDisconnect[iClient])
	{
		WriteLogEx(pSockSrvr->icClientID[iClient], "File Transfer complete.", EVENT_NONE);
		pSockSrvr->SetNextSendData(iClient, "::EOF");

		if(bSendCRC)
		{
			//Send the data CRC.
			sprintf(sFileBuffer, "::DataCRC->%d", dwDataCRC);
			pSockSrvr->SetNextSendData(iClient, sFileBuffer);
		}
		return true;
	}
	else return false;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int ExecuteTransactionSQL(CSockSrvr *pSockSrvr, int iClient)
{
	char sTempFile[MAX_PATH];
    char sSendBuf[SENDBUFSZ + 1];

	int iSendBufSz = 0;
	long lFetchResult = 0;

	char sStatement[2048 + 1];
	char sImportTable[2048 + 1];
	char sImportDB[2048 + 1];
	int iSz = 0;

	if(gbDebugMode)
	{
		WriteLogEx(pSockSrvr->icClientID[iClient], "Executing transaction SQL.", EVENT_NONE);
	}

	while(CCI.rsTrans.FetchEx(&lFetchResult))
	{
		CCI.rsTrans.sColumnEx(1, sStatement, sizeof(sStatement), &iSz);
		if(iSz <= 0)
		{
			strcpy(sStatement, "");
		}

		CCI.rsTrans.sColumnEx(2, CCI.sOnSuccess, sizeof(CCI.sOnSuccess), &iSz);
		if(iSz <= 0)
		{
			strcpy(CCI.sOnSuccess, "");
		}

		CCI.rsTrans.sColumnEx(3, CCI.sOnFailure, sizeof(CCI.sOnFailure), &iSz);
		if(iSz <= 0)
		{
			strcpy(CCI.sOnFailure, "");
		}

		CCI.rsTrans.sColumnEx(4, sImportDB, sizeof(sImportDB), &iSz);
		if(iSz <= 0)
		{
			strcpy(sImportDB, "");
		}
		
		CCI.rsTrans.sColumnEx(5, sImportTable, sizeof(sImportTable), &iSz);
		if(iSz <= 0)
		{
			strcpy(sImportTable, "");
		}
		
		if(iSz > 0)
		{
			GetTempFileName(sTempFilesPath, "SEC", 0, sTempFile);

			int iSQLExpRes = ExportSQLResults(pSockSrvr, iClient, sStatement,
				sImportDB, gsDefaultDBO, sImportTable, sTempFile);

			if(iSQLExpRes == SQL_IMPORT_RESULT_ZEROROWS)
			{
				//The statement was supposed to return data, it didnt... Thats ok!
				//Process the next statement.
				DeleteFile(sTempFile);
				continue;
			}
			else if(iSQLExpRes == SQL_IMPORT_RESULT_OK){
				//sprintf(FlaggedData, "::FileSize:%d", Get_FileSize(gsSQLExportTempFile));
				//SimpleClientSend(FlaggedData);

				sprintf(sSendBuf, "::SendingFileData->%s", sImportTable);
				pSockSrvr->SetNextSendData(iClient, sSendBuf);

				SendFileData(pSockSrvr, iClient, sTempFile, true);
				return CMD_OK;
			}
			else if(iSQLExpRes == SQL_IMPORT_RESULT_ERROR){
				// There was an error exporting the SQL data.
				// FIXME: What do we do now, Process the next statement or disconnect?
				//		Need to think over the logic.
				DeleteFile(sTempFile);
				return CMD_ERROR;
			}
		}
		else{
			//The statement was not a statement that is supposed to return data.
			//Process the next statement.

			CRecordSet rsTemp;

			if(!CCI.cCustSQL.Execute(sStatement, &rsTemp))
			{
				sprintf(sSendBuf, "Execute: Fail on (%s)", sStatement);
				WriteLogEx(pSockSrvr->icClientID[iClient], sSendBuf, EVENT_ERROR);
				return CMD_ERROR;
			}

			if(rsTemp.RowCount > 0)
			{
				sprintf(sSendBuf, "Statement affected %d rows.", rsTemp.RowCount);
				WriteLogEx(pSockSrvr->icClientID[iClient], sSendBuf, EVENT_NONE);
			}

			rsTemp.Close();
			continue;
		}
	}

	if(lFetchResult == SQL_NO_DATA_FOUND)
	{
		//Need to delte all transaction from the "SQLExch_Trans" table
		//	where the "SQLExch_Trans.SQLExch_Pending" flag is set to 1.
		//	Any transactions that were added durring the replication process will not
		//	be deleted.
		sprintf(sStatement, "DELETE FROM [%s].[%s].SQLExch_Trans"
			" WHERE SQLExch_Pending = 1",
			gsReplicationDB, gsDefaultDBO);
		CCI.cCustSQL.ExecuteNonQuery(sStatement);

		WriteLogEx(pSockSrvr->icClientID[iClient], "End of statements.", EVENT_NONE);
		pSockSrvr->SetNextSendData(iClient, "::Complete");
		CCI.rsTrans.Close();

		WriteLogEx(pSockSrvr->icClientID[iClient], "The process has been completed successfully.", EVENT_NONE);

		return CMD_OK;
	}
	else{
		sprintf(sSendBuf, "SQL Fetch error: %d.", lFetchResult);
		WriteLogEx(pSockSrvr->icClientID[iClient], sSendBuf, EVENT_ERROR);
		return CMD_ERROR;
	}

	return CMD_ERROR;
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif
